@@ -10,30 +10,35 @@
     :license: BSD, see LICENSE for more details.
 """
 
-nodeenv_version = '0.12.2'
-
+import contextlib
+import io
 import sys
 import os
 import re
 import stat
 import logging
+import operator
 import optparse
 import subprocess
+import tarfile
 import pipes
 
 try:  # pragma: no cover (py2 only)
     from ConfigParser import SafeConfigParser as ConfigParser
     from HTMLParser import HTMLParser
     from urllib2 import urlopen
-    iteritems = lambda dict_: dict_.iteritems()
+    iteritems = operator.methodcaller('iteritems')
 except ImportError:  # pragma: no cover (py3 only)
     from configparser import ConfigParser
     from html.parser import HTMLParser
     from urllib.request import urlopen
-    iteritems = lambda dict_: dict_.items()
+    iteritems = operator.methodcaller('items')
 
 from pkg_resources import parse_version
 
+
+nodeenv_version = '0.13.0'
+
 join = os.path.join
 abspath = os.path.abspath
 src_domain = "nodejs.org"
@@ -441,33 +446,24 @@ def get_node_src_url(version, postfix=''):
     return node_url
 
 
+@contextlib.contextmanager
+def tarfile_open(*args, **kwargs):
+    """Compatibility layer because py26."""
+    tf = tarfile.open(*args, **kwargs)
+    try:
+        yield tf
+    finally:
+        tf.close()
+
+
 def download_node(node_url, src_dir, env_dir, opt):
     """
     Download source code
     """
-    cmd = []
-    cmd.append('curl')
-    cmd.append('--silent')
-    cmd.append('-L')
-    cmd.append(node_url)
-    cmd.append('|')
-    cmd.append('tar')
-    cmd.append('xzf')
-    cmd.append('-')
-    cmd.append('-C')
-    cmd.append(pipes.quote(src_dir))
-    cmd.extend(['--exclude', 'ChangeLog'])
-    cmd.extend(['--exclude', 'LICENSE'])
-    cmd.extend(['--exclude', 'README.md'])
-    try:
-        callit(cmd, opt.verbose, True, env_dir)
-        logger.info(') ', extra=dict(continued=True))
-    except OSError:
-        postfix = '-RC1'
-        logger.info('%s) ' % postfix, extra=dict(continued=True))
-        new_node_url = get_node_src_url(opt.node, postfix)
-        cmd[cmd.index(node_url)] = new_node_url
-        callit(cmd, opt.verbose, True, env_dir)
+    tar_contents = io.BytesIO(urlopen(node_url).read())
+    with tarfile_open(fileobj=tar_contents) as tarfile_obj:
+        tarfile_obj.extractall(src_dir)
+    logger.info(')', extra=dict(continued=True))
 
 
 def get_node_src_url_postfix(opt):
@@ -587,16 +583,27 @@ def install_npm(env_dir, src_dir, opt):
     """
     logger.info(' * Install npm.js (%s) ... ' % opt.npm,
                 extra=dict(continued=True))
-    cmd = [
-        '. %s && curl --location --silent %s | '
-        'clean=%s npm_install=%s bash && deactivate_node' % (
-            pipes.quote(join(env_dir, 'bin', 'activate')),
-            'https://www.npmjs.org/install.sh',
-            'no' if opt.no_npm_clean else 'yes',
-            opt.npm
-        )
-    ]
-    callit(cmd, opt.verbose, True)
+    npm_contents = urlopen('https://www.npmjs.org/install.sh').read()
+    env = dict(
+        os.environ,
+        clean='no' if opt.no_npm_clean else 'yes',
+        npm_install=opt.npm,
+    )
+    proc = subprocess.Popen(
+        (
+            'bash', '-c',
+            '. {0} && exec bash'.format(
+                pipes.quote(join(env_dir, 'bin', 'activate')),
+            )
+        ),
+        env=env,
+        stdin=subprocess.PIPE,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.STDOUT,
+    )
+    out, _ = proc.communicate(npm_contents)
+    if opt.verbose:
+        logger.info(out)
     logger.info('done.')
 
 
@@ -704,28 +711,32 @@ def create_environment(env_dir, opt):
         callit(['rm -rf', pipes.quote(src_dir)], opt.verbose, True, env_dir)
 
 
+VERSION_RE = re.compile('\d+\.\d+\.\d+')
+
+
+def get_node_versions():
+    response = urlopen('https://{0}/dist'.format(src_domain))
+    href_parser = GetsAHrefs()
+    href_parser.feed(response.read().decode('UTF-8'))
+    versions = set(
+        VERSION_RE.search(href).group()
+        for href in href_parser.hrefs
+        if VERSION_RE.search(href)
+    )
+    sorted_versions = sorted([parse_version(version) for version in versions])
+    return [v.public for v in sorted_versions]
+
+
 def print_node_versions():
     """
     Prints into stdout all available node.js versions
     """
-    p = subprocess.Popen(
-        ("curl -s https://%s/dist/ | "
-         "egrep -o '[0-9]+\.[0-9]+\.[0-9]+' | "
-         "sort -u -k 1,1n -k 2,2n -k 3,3n -t . ") % (src_domain),
-        shell=True, stdout=subprocess.PIPE)
-    # out, err = p.communicate()
-    pos = 0
-    rowx = []
-    while 1:
-        row = p.stdout.readline()
-        pos += 1
-        if not row:
-            logger.info('\t'.join(rowx))
-            break
-        rowx.append(row.decode('utf-8').replace('\n', ''))
-        if pos % 8 == 0:
-            logger.info('\t'.join(rowx))
-            rowx = []
+    versions = get_node_versions()
+    chunks_of_8 = [
+        versions[pos:pos + 8] for pos in range(0, len(versions), 8)
+    ]
+    for chunk in chunks_of_8:
+        logger.info('\t'.join(chunk))
 
 
 class GetsAHrefs(HTMLParser):
@@ -793,12 +804,6 @@ def main():
         Config._dump()
         return
 
-    for exe in ('curl', 'egrep', 'sort', 'tar'):
-        if not is_installed(exe):
-            print 'Error: "%s" not installed.' % exe
-            print 'Please, install it via apt/yum/etc and try again.'
-            return sys.exit(1)
-
     opt, args = parse_args(check=False)
     Config._load(opt.config_file, opt.verbose)
 
